﻿using Microsoft.Win32;
using MOFT.Classes;
using MOFT.Windows.DailyReport;
using MOFT.Windows.Diagnostics;
using MOFT.Windows.Export;
using MOFT.Windows.Lists;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Markup;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Xaml.Permissions;
using Xceed.Wpf.Toolkit.Primitives;

namespace MOFT
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public string SelectedLanguage = "ENGLISH";       
        public string Culture = "en-US";
        public string TemperatureUnit = "CELSIUS";
        public static string AppExecutablePath = System.Diagnostics.Process.GetCurrentProcess().MainModule.FileName;
        public static string appDirectory = System.IO.Path.GetDirectoryName(System.Diagnostics.Process.GetCurrentProcess().MainModule.FileName);
        public string FullFilePath = "";
        public string ProjectName = "";
        private int lastSelectedSearchIndex = -1;
        private AquariumViewModel viewModel = new AquariumViewModel();

        #region Paths

        public string pathFishList = appDirectory + "\\Data\\Species.txt";
        public string pathOriginList = appDirectory + "\\Data\\Origin.txt";
        public string pathDiseasesList = appDirectory + "\\Data\\Diseases.txt";
        public string pathSupplierList = appDirectory + "\\Data\\Supplier.txt";
        public string pathMedicineList = appDirectory + "\\Data\\Medicine.txt";

        #endregion

        #region Lists

        public List<Aquarium> AquariumList = new List<Aquarium>();
        public List<string> FishList = new List<string>();
        public List<string> DiseaseList = new List<string>();
        public List<string> OriginList = new List<string>();
        public List<string> SupplierList = new List<string>();
        public List<string> MedicineList = new List<string>();

        #endregion

        public MainWindow(string path = "")
        {

            ReadRegistry();
            LoadLanguageResource();
            SetCulture();
            InitializeComponent();
            LoadExternalData();
            SelectMenuItem(LANGUAGE_MENU_ITEM, SelectedLanguage);
            SelectMenuItem(MENU_ITEM_TEMPERATURE, TemperatureUnit);
            SetUIStrings();

            if (!string.IsNullOrEmpty(path))
            {
                FullFilePath = path;
                StartProject(FullFilePath);
            }         
            DataContext = viewModel;
        }

        #region Start

        private void LoadExternalData()
        {
            CreateFileIfNotExist(pathFishList);
            CreateFileIfNotExist(pathOriginList);
            CreateFileIfNotExist(pathDiseasesList);
            CreateFileIfNotExist(pathSupplierList);
            CreateFileIfNotExist(pathMedicineList);

            LoadStringList(pathFishList, FishList);
            LoadStringList(pathOriginList, OriginList);
            LoadStringList(pathDiseasesList, DiseaseList);
            LoadStringList(pathSupplierList, SupplierList);
            LoadStringList(pathMedicineList, MedicineList);

            cb_FishSpecies.ItemsSource = FishList;
            cb_FishOrigin.ItemsSource = OriginList;
            cb_FishSupplier.ItemsSource = SupplierList;
            cb_Medicine.ItemsSource = MedicineList;
            cb_Disease.ItemsSource = DiseaseList;
        }

        private void CreateFileIfNotExist(string path)
        {
            if (!File.Exists(path))
            {
                File.Create(path).Close(); ;
            }
        }

        private void LoadStringList(string path, List<string> listString)
        {
            try
            {
                using (StreamReader sr = new StreamReader(path))
                {
                    string line;
                    while ((line = sr.ReadLine()) != null)
                    {
                        listString.Add(line);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Warning", MessageBoxButton.OK, MessageBoxImage.Warning);
            }
        }

        private void ReadRegistry()
        {
            SelectedLanguage = GetRegistryValue("Language", "ENGLISH");
            TemperatureUnit = GetRegistryValue("TemperatureUnit", "CELSIUS");
        }   

        private void SetUIStrings()
        {

            lbl_Temperature.Content = (TryFindResource("STR_TEMPERATURE").ToString() + " (" + (TemperatureUnit == "CELSIUS" ? "°C" : "°F") + "):");
        }

        private void LoadLanguageResource()
        {
            try
            {
                string resourceFileName = $"Resources/Resource_{SelectedLanguage.ToUpper()}.xaml";

                var dict = new ResourceDictionary();
                dict.Source = new Uri(resourceFileName, UriKind.Relative);

                Application.Current.Resources.MergedDictionaries.Clear();
                Application.Current.Resources.MergedDictionaries.Add(dict);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void SetCulture()
        {
            // Default culture
           Culture = "en-US";

            switch (SelectedLanguage.ToUpper())
            {
                case "CZECH":
                    Culture = "cs-CZ";
                    break;
                case "GERMAN":
                    Culture = "de-DE";
                    break;
                case "INDONESIAN":
                    Culture = "id-ID";
                    break;
            }

            CultureInfo cultureInfo = new CultureInfo(Culture);
            Thread.CurrentThread.CurrentCulture = cultureInfo;
            Thread.CurrentThread.CurrentUICulture = cultureInfo;
            FrameworkElement.LanguageProperty.OverrideMetadata(
                typeof(FrameworkElement),
                new FrameworkPropertyMetadata(
                    XmlLanguage.GetLanguage(CultureInfo.CurrentCulture.IetfLanguageTag)));
        }

        private string GetRegistryValue(string valueName, string defaultValue)
        {
            string registryKeyPath = "Software\\MOFT";

            try
            {
                using (var key = Registry.CurrentUser.OpenSubKey(registryKeyPath, true) ?? Registry.CurrentUser.CreateSubKey(registryKeyPath))
                {
                    var value = key.GetValue(valueName)?.ToString();

                    // If the value doesn't exist, set the default value
                    if (value == null)
                    {
                        key.SetValue(valueName, defaultValue, RegistryValueKind.String);
                        return defaultValue;
                    }

                    return value;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(TryFindResource("STR_REGISTRY_READ_ERROR") + ex.Message);
            }

            return defaultValue;
        }

        #endregion

        #region LanguageSelect

        private void LanguageMenuItem_Click(object sender, RoutedEventArgs e)
        {
            MenuItem menuItem = sender as MenuItem;
            if (menuItem != null)
            {
                string[] nameParts = menuItem.Name.Split('_');
                string selectedLanguage = nameParts[nameParts.Length - 1];

                try
                {
                    WriteRegistryValue("Language", selectedLanguage.ToUpper());
                    string confirmationMessage = TryFindResource("STR_RESTART_CONFIRM").ToString();
                    MessageBoxResult confirmationResult = MessageBox.Show(confirmationMessage, TryFindResource("STR_CONFIRMATION").ToString(), MessageBoxButton.YesNo, MessageBoxImage.Warning);
                    if (confirmationResult == MessageBoxResult.Yes)
                    {
                        RestartApplication();
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error: " + ex.Message);
                }
            }
        }

        private void SelectMenuItem(MenuItem m, string text)
        {
            foreach (MenuItem menuItem in m.Items)
            {
                if (menuItem.Name.EndsWith(text, StringComparison.OrdinalIgnoreCase))
                {
                    menuItem.IsChecked = true;
                }
                else
                {
                    menuItem.IsChecked = false;
                }
            }
        }

        private void WriteRegistryValue(string valueName, string value)
        {
            try
            {
                string registryKeyPath = "Software\\MOFT";

                using (RegistryKey key = Registry.CurrentUser.CreateSubKey(registryKeyPath))
                {
                    key.SetValue(valueName, value, RegistryValueKind.String);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(TryFindResource("STR_REGISTRY_WRITE_ERROR") + ex.Message);
            }
        }

        private void RestartApplication()
        {
            Application.Current.Shutdown();
            System.Diagnostics.Process.Start(AppExecutablePath);
        }

        #endregion

        #region KeyDownEvent

        private void MainWindow_KeyDown(object sender, KeyEventArgs e)
        {
            //CTRL+N
            if (e.Key == Key.N && (Keyboard.Modifiers & ModifierKeys.Control) == ModifierKeys.Control)
            {
                NewProject_Click(sender, e);
            }

            //CTRL+O
            if (e.Key == Key.O && (Keyboard.Modifiers & ModifierKeys.Control) == ModifierKeys.Control)
            {
                OpenProject_Click(sender, e);
            }

            //CTRL+E
            if (e.Key == Key.E && (Keyboard.Modifiers & ModifierKeys.Control) == ModifierKeys.Control)
            {
                ExportProject_Click(sender, e);
            }

            //CTRL+S
            if (e.Key == Key.S && (Keyboard.Modifiers & ModifierKeys.Control) == ModifierKeys.Control)
            {
                SaveProject_Click(sender, e);
            }

            // Ctrl + Shift + S
            if (e.Key == Key.S && Keyboard.Modifiers == (ModifierKeys.Control | ModifierKeys.Shift))
            {
                SaveProjectAs_Click(sender, e);
            }
        }

        #endregion

        #region NewProject

        private void NewProject_Click(object sender, RoutedEventArgs e)
        {
            NewProjectWindow newProjectWindow = new NewProjectWindow(this);
            newProjectWindow.Show();
        }

        #endregion

        #region OpenProject

        private void OpenProject_Click(object sender, RoutedEventArgs e)
        {
            string defaultDirectory = System.IO.Path.Combine(appDirectory, "Projects");

            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = "MOFT files (*.moft)|*.moft|All files (*.*)|*.*";
            openFileDialog.FilterIndex = 1;
            openFileDialog.RestoreDirectory = true;
            openFileDialog.InitialDirectory = defaultDirectory;

            if (openFileDialog.ShowDialog() == true)
            {
                FullFilePath = openFileDialog.FileName;
                StartProject(FullFilePath);
            }
        }

        #endregion

        #region StartProject

        internal void StartProject(string path)
        {
            AquariumList.Clear();
            XML.LoadProject(path, AquariumList);
            if (TemperatureUnit != XML.temperatureUnit)
            {
                foreach (Aquarium a in AquariumList)
                {
                    a.fTemperature = TemperatureUnit == "CELSIUS" ? FahrenheitToCelsius(a.fTemperature) : CelsiusToFahrenheit(a.fTemperature);
                }
            }
            LoadListView();
            ShowUI(true);

            if (AquariumList.Count > 0)
                aquariumListView.SelectedIndex = 0;

            ProjectName = System.IO.Path.GetFileNameWithoutExtension(path);
            MessageBox.Show(TryFindResource("STR_PROJECT").ToString() + " " + ProjectName + " " + TryFindResource("STR_PROJECT_LOADED").ToString());

            Title = "MOFT - " + path;
        }

        private void LoadListView()
        {
            aquariumListView.Items.Clear();
            foreach (Aquarium a in AquariumList)
            {
                aquariumListView.Items.Add(a);
            }
        }

        private void ShowUI(bool show)
        {
            Main_Grid.Background = null;
            SolidColorBrush s = new SolidColorBrush(Color.FromArgb(255, 202, 213, 216));
            Background_Grid.Background = s;
            Search_Grid.Visibility = show ? Visibility.Visible : Visibility.Hidden;
            aquariumListView.Visibility = show ? Visibility.Visible : Visibility.Hidden;
            grid_splitter.Visibility = show ? Visibility.Visible : Visibility.Hidden;
            tc_Main.Visibility = show ? Visibility.Visible : Visibility.Hidden;
            Legend_Grid.Visibility = show ? Visibility.Visible : Visibility.Hidden;
            Footer_Grid.Visibility = show ? Visibility.Visible : Visibility.Hidden;
        }
        #endregion

        #region ExportProject

        private void ExportProject_Click(object sender, RoutedEventArgs e)
        {
            ExportDialog exportDialog = new ExportDialog(this);
            exportDialog.ShowDialog();
        }

        #endregion

        #region Saving

        private void SaveProject_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(FullFilePath))
                {
                    MessageBox.Show(
                        TryFindResource("STR_PROJECT_NOT_OPEN").ToString(),
                        TryFindResource("STR_ERROR").ToString(),
                        MessageBoxButton.OK,
                        MessageBoxImage.Error
                    );
                    return;
                }

                XML.WriteFileStart(FullFilePath, Culture, TemperatureUnit);

                foreach (Aquarium a in AquariumList)
                {
                    XML.WriteAquarium(a);
                }

                XML.WriteFileEnd();

                MessageBox.Show(
                    $"{TryFindResource("STR_PROJECT")} {ProjectName} {TryFindResource("STR_SAVED")}",
                    TryFindResource("STR_PROJECT_SAVED").ToString(),
                    MessageBoxButton.OK,
                    MessageBoxImage.Information
                );
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, TryFindResource("STR_ERROR").ToString(), MessageBoxButton.OK, MessageBoxImage.Error);
            }       
        }

        private void SaveProjectAs_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                SaveFileDialog saveFileDialog = new SaveFileDialog
                {
                    Filter = "MOFT Files (*.moft)|*.moft",
                    DefaultExt = "moft",
                    Title = TryFindResource("STR_SAVE_PROJECT_AS").ToString()
                };

                if (saveFileDialog.ShowDialog() == true)
                {
                    FullFilePath = saveFileDialog.FileName;

                    XML.WriteFileStart(FullFilePath, Culture, TemperatureUnit);

                    foreach (Aquarium a in AquariumList)
                    {
                        XML.WriteAquarium(a);
                    }

                    XML.WriteFileEnd();

                    MessageBox.Show(
                        $"{TryFindResource("STR_PROJECT")} {System.IO.Path.GetFileNameWithoutExtension(FullFilePath)} {TryFindResource("STR_SAVED")}",
                        TryFindResource("STR_PROJECT_SAVED").ToString(),
                        MessageBoxButton.OK,
                        MessageBoxImage.Information
                    );
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, TryFindResource("STR_ERROR").ToString(), MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        #endregion

        #region ClearUIFunctions

        private void ClearSpeciesUI()
        {            
            cb_FishSpecies.SelectedIndex = -1;
            cb_SpeciesNumber.SelectedItem = -1;
            tb_FishCode.Text = string.Empty;
            cb_FishSex.SelectedItem = null;
            tb_FishLength.Text = string.Empty;
            tb_FishStockQuantity.Text = string.Empty;
            tb_FishCurrentQuantity.Text = string.Empty;
            cb_FishOrigin.SelectedIndex = -1;
            cb_FishSupplier.SelectedIndex = -1;
            dp_FishImport.SelectedDate = DateTime.Today;
            tb_FishOwnNotes.Text = string.Empty;
            tb_FishMortality.Text = string.Empty;
        //  ClearDiseaseUI();
        }

        private void ClearDiseaseUI()
        {
            cb_DiseaseNumber.SelectedIndex = -1;
            cb_Disease.SelectedIndex = -1;
            dp_DateOfDiagnosis.SelectedDate = DateTime.Today;          
            ClearMedicineUI();
        }

        private void ClearMedicineUI()
        {
            cb_Medicine.SelectedIndex = -1;
            cb_MedicineNumber.SelectedIndex = -1;          
            tb_Dosage100L.Text = string.Empty;
            dp_LastDosage.SelectedDate = DateTime.Today;
            tb_DosagePeriod.Text = string.Empty;
            tb_DosageCurrent.Text = string.Empty;
            tb_NextDosage.Text = string.Empty;
        }

        private void EnableDiseaseUI(bool bEnable)
        {
            cb_Disease.IsEnabled = bEnable;
            dp_DateOfDiagnosis.IsEnabled = bEnable;
            cb_Medicine.IsEnabled = bEnable;
            cb_MedicineNumber.IsEnabled = bEnable;
            tb_Dosage100L.IsEnabled = bEnable;
            dp_LastDosage.IsEnabled = bEnable;
            tb_DosagePeriod.IsEnabled = bEnable;
            tb_DosageCurrent.IsEnabled = bEnable;
            tb_NextDosage.IsEnabled = bEnable;
            cb_Dosage_Measure_Unit.IsEnabled = bEnable;
        }

        #endregion

        #region AddingDeleting

        #region SpeciesAddingDeleting

        private void btn_Add_Species_Click(object sender, RoutedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                if (selectedAquarium.bWaterPresent != 1 || chckbx_WaterPresent.IsChecked == false)
                {
                    MessageBox.Show(TryFindResource("STR_ERROR_NO_WATER").ToString());
                    return;
                }
                if (selectedAquarium.vFishSpecies == null)
                {
                    selectedAquarium.vFishSpecies = new ObservableCollection<FishSpecies>();
                }

                FishSpecies newSpecies = new FishSpecies();

                selectedAquarium.vFishSpecies.Add(newSpecies);
                selectedAquarium.bSpeciesCount++;
                btn_Delete_Species.Visibility = Visibility.Visible;
                BindingExpression bindingExpression = cb_SpeciesNumber.GetBindingExpression(ComboBox.ItemsSourceProperty);
                bindingExpression?.UpdateTarget();
                bindingExpression = cb_DiseaseSpecies.GetBindingExpression(ComboBox.ItemsSourceProperty);
                bindingExpression?.UpdateTarget();
                cb_SpeciesNumber.SelectedIndex = selectedAquarium.bSpeciesCount - 1;
                cb_FishSpecies.SelectedItem = null;
            }
            else
            {
                MessageBox.Show(TryFindResource("STR_SELECT_AQUARIUM_FIRST").ToString());
            }
        }

        private void btn_Delete_Species_Click(object sender, RoutedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                selectedAquarium.vFishSpecies.RemoveAt(cb_SpeciesNumber.SelectedIndex);
                selectedAquarium.bSpeciesCount--;
                if (selectedAquarium.bSpeciesCount == 0)
                {
                    btn_Delete_Species.Visibility = Visibility.Hidden;
                    selectedAquarium.bHasDisease = false;
                    selectedAquarium.bHasDosageWarning = false;
                    ClearSpeciesUI();
                }
                BindingExpression bindingExpression = cb_SpeciesNumber.GetBindingExpression(ComboBox.ItemsSourceProperty);
                bindingExpression?.UpdateTarget();
                bindingExpression = cb_DiseaseSpecies.GetBindingExpression(ComboBox.ItemsSourceProperty);
                bindingExpression?.UpdateTarget();
                cb_SpeciesNumber.SelectedIndex = selectedAquarium.bSpeciesCount - 1;
            }
            else
            {
                MessageBox.Show(TryFindResource("STR_SELECT_AQUARIUM_FIRST").ToString());
            }
        }

        #endregion

        #region DiseaseAddingDeleting

        private void btn_Add_Disease_Click(object sender, RoutedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                if (cb_DiseaseSpecies.SelectedItem != null && cb_DiseaseSpecies.SelectedIndex >= 0 && cb_DiseaseSpecies.SelectedIndex < selectedAquarium.vFishSpecies.Count)
                {
                    FishSpecies selectedSpecies = selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex];

                    if (selectedSpecies.vDiseases == null)
                    {
                        selectedSpecies.vDiseases = new ObservableCollection<Disease>();
                    }

                    Disease newDisease = new Disease();
                    selectedSpecies.vDiseases.Add(newDisease);
                    selectedSpecies.bDiseaseCount++;
                    selectedAquarium.bHasDisease = true;
                    btn_Delete_Disease.Visibility = Visibility.Visible;
                    BindingExpression bindingExpression = cb_DiseaseNumber.GetBindingExpression(ComboBox.ItemsSourceProperty);
                    bindingExpression?.UpdateTarget();
                    cb_DiseaseNumber.SelectedIndex = selectedSpecies.vDiseases.Count - 1;
                }
                else
                {
                    MessageBox.Show(TryFindResource("STR_SELECT_SPECIES_FIRST").ToString());
                }
            }
            else
            {
                MessageBox.Show(TryFindResource("STR_SELECT_AQUARIUM_FIRST").ToString());
            }
        }

        private void btn_Delete_Disease_Click(object sender, RoutedEventArgs e)
        {

            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                if (cb_DiseaseSpecies.SelectedItem != null && cb_DiseaseSpecies.SelectedIndex >= 0 && cb_DiseaseSpecies.SelectedIndex < selectedAquarium.vFishSpecies.Count)
                {
                    FishSpecies selectedSpecies = selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex];

                    if (cb_DiseaseNumber.SelectedItem != null && cb_DiseaseNumber.SelectedIndex >= 0 && cb_DiseaseNumber.SelectedIndex < selectedSpecies.vDiseases.Count)
                    {
                        selectedSpecies.vDiseases.RemoveAt(cb_DiseaseNumber.SelectedIndex);
                        selectedSpecies.bDiseaseCount--;
                        if (selectedSpecies.bDiseaseCount == 0)
                        {
                            btn_Delete_Disease.Visibility = Visibility.Hidden;
                            viewModel.SelectedDisease = null;
                            ClearDiseaseUI();
                        }

                        CheckAquariumProperty(selectedAquarium);
                        BindingExpression bindingExpression = cb_DiseaseNumber.GetBindingExpression(ComboBox.ItemsSourceProperty);
                        bindingExpression?.UpdateTarget();
                        cb_DiseaseNumber.SelectedIndex = selectedSpecies.bDiseaseCount - 1;
                    }
                }
                else
                {
                    MessageBox.Show(TryFindResource("STR_SELECT_SPECIES_FIRST").ToString());
                }
            }
            else
            {
                MessageBox.Show(TryFindResource("STR_SELECT_AQUARIUM_FIRST").ToString());
            }
        }

        #endregion

        #region MedicineAddingDeleting

        private void btn_Add_Medicine_Click(object sender, RoutedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                if (cb_SpeciesNumber.SelectedItem != null && cb_SpeciesNumber.SelectedIndex >= 0 && cb_SpeciesNumber.SelectedIndex < selectedAquarium.vFishSpecies.Count)
                {
                    FishSpecies selectedSpecies = selectedAquarium.vFishSpecies[cb_SpeciesNumber.SelectedIndex];

                    if (cb_DiseaseNumber.SelectedItem != null && cb_DiseaseNumber.SelectedIndex >= 0 && cb_DiseaseNumber.SelectedIndex < selectedSpecies.vDiseases.Count)
                    {
                        Disease selectedDisease = selectedSpecies.vDiseases[cb_DiseaseNumber.SelectedIndex];

                        if (selectedDisease.vMedicines == null)
                        {
                            selectedDisease.vMedicines = new ObservableCollection<Medicine>();
                        }

                        Medicine newMedicine = new Medicine();
                        selectedDisease.vMedicines.Add(newMedicine);
                        selectedDisease.bMedicineCount++;
                        btn_Delete_Medicine.Visibility = Visibility.Visible;
                        BindingExpression bindingExpression = cb_MedicineNumber.GetBindingExpression(ComboBox.ItemsSourceProperty);
                        bindingExpression?.UpdateTarget();
                        cb_MedicineNumber.SelectedIndex = selectedDisease.vMedicines.Count - 1;
                    }
                    else
                    {
                        MessageBox.Show(TryFindResource("STR_SELECT_DISEASE_FIRST").ToString());
                    }
                }
                else
                {
                    MessageBox.Show(TryFindResource("STR_SELECT_SPECIES_FIRST").ToString());
                }
            }
            else
            {
                MessageBox.Show(TryFindResource("STR_SELECT_AQUARIUM_FIRST").ToString());
            }
        }

        private void btn_Delete_Medicine_Click(object sender, RoutedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                if (cb_SpeciesNumber.SelectedItem != null && cb_SpeciesNumber.SelectedIndex >= 0 && cb_SpeciesNumber.SelectedIndex < selectedAquarium.vFishSpecies.Count)
                {
                    FishSpecies selectedSpecies = selectedAquarium.vFishSpecies[cb_SpeciesNumber.SelectedIndex];

                    if (cb_DiseaseNumber.SelectedItem != null && cb_DiseaseNumber.SelectedIndex >= 0 && cb_DiseaseNumber.SelectedIndex < selectedSpecies.vDiseases.Count)
                    {
                        Disease selectedDisease = selectedSpecies.vDiseases[cb_DiseaseNumber.SelectedIndex];

                        if (selectedDisease.vMedicines != null && selectedDisease.vMedicines.Count > 0)
                        {
                            selectedDisease.vMedicines.RemoveAt(cb_MedicineNumber.SelectedIndex);
                            selectedDisease.bMedicineCount--;
                            if (selectedDisease.bMedicineCount == 0)
                            {
                                btn_Delete_Medicine.Visibility = Visibility.Hidden;
                                ClearMedicineUI();
                            }
                            BindingExpression bindingExpression = cb_MedicineNumber.GetBindingExpression(ComboBox.ItemsSourceProperty);
                            bindingExpression?.UpdateTarget();
                            cb_MedicineNumber.SelectedIndex = selectedDisease.bMedicineCount - 1;
                        }
                    }
                    else
                    {
                        MessageBox.Show(TryFindResource("STR_SELECT_DISEASE_FIRST").ToString());
                    }
                }
                else
                {
                    MessageBox.Show(TryFindResource("STR_SELECT_SPECIES_FIRST").ToString());
                }
            }
            else
            {
                MessageBox.Show(TryFindResource("STR_SELECT_AQUARIUM_FIRST").ToString());
            }
        }

        #endregion

        #endregion

        #region SelectionsChanged

        private void aquariumListView_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                viewModel.SelectedAquarium = selectedAquarium;
                tc_Main.SelectedIndex = 0;
                chckbx_WaterPresent.IsChecked = selectedAquarium.bWaterPresent != 0;
                tb_AquariumNextWaterChange.Foreground = (selectedAquarium.dNextWaterChange <= DateTime.Today) ? Brushes.Red : Brushes.Black;
                tb_AquariumNextFilterClean.Foreground = (selectedAquarium.dNextFilterClean <= DateTime.Today) ? Brushes.Red : Brushes.Black;

                if (selectedAquarium.bSpeciesCount > 0)
                {
                    btn_Delete_Species.Visibility = Visibility.Visible;
                    cb_SpeciesNumber.SelectedIndex = 0;
                }
                else
                {
                    btn_Delete_Species.Visibility = Visibility.Hidden;
                    viewModel.SelectedFishSpecies = null;
                    viewModel.SelectedDisease = null;
                    viewModel.SelectedMedicine = null;
                }
            }
        }

        private void cb_SpeciesNumber_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium &&
                selectedAquarium.vFishSpecies != null && selectedAquarium.vFishSpecies.Count > 0 &&
                cb_SpeciesNumber.SelectedIndex >= 0 && cb_SpeciesNumber.SelectedIndex < selectedAquarium.vFishSpecies.Count)
            {
                FishSpecies selectedSpecies = selectedAquarium.vFishSpecies[cb_SpeciesNumber.SelectedIndex];
                viewModel.SelectedFishSpecies = selectedSpecies;
                cb_DiseaseSpecies.SelectedIndex = cb_SpeciesNumber.SelectedIndex;
            }
        }

        private void cb_FishSpecies_SelectionChanged(object sender, SelectionChangedEventArgs e) 
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                if (selectedAquarium.vFishSpecies != null)
                {
                    int selected = cb_SpeciesNumber.SelectedIndex;
                    BindingExpression bindingExpression = cb_SpeciesNumber.GetBindingExpression(ComboBox.ItemsSourceProperty);
                    bindingExpression?.UpdateTarget();
                    bindingExpression = cb_DiseaseSpecies.GetBindingExpression(ComboBox.ItemsSourceProperty);
                    bindingExpression?.UpdateTarget();
                    cb_SpeciesNumber.SelectedIndex = selected;
                } 
            }
        }

        private void cb_DiseaseSpecies_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium &&
                selectedAquarium.vFishSpecies != null && selectedAquarium.vFishSpecies.Count > 0 &&
                cb_DiseaseSpecies.SelectedIndex >= 0 && cb_DiseaseSpecies.SelectedIndex < selectedAquarium.vFishSpecies.Count)
            {
                FishSpecies selectedSpecies = selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex];
                cb_SpeciesNumber.SelectedIndex = cb_DiseaseSpecies.SelectedIndex;
                viewModel.SelectedFishSpecies = selectedSpecies;

                if (selectedSpecies.vDiseases.Count > 0)
                {
                    cb_DiseaseNumber.SelectedIndex = 0;                  
                }
                else
                {
                    viewModel.SelectedDisease = null;
                    viewModel.SelectedMedicine = null;
                }
            }
        }      

        private void cb_DiseaseNumber_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                if (cb_DiseaseSpecies.SelectedItem != null && cb_DiseaseSpecies.SelectedIndex >= 0 && cb_DiseaseSpecies.SelectedIndex < selectedAquarium.vFishSpecies.Count)
                {
                    FishSpecies selectedSpecies = selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex];

                    if (selectedSpecies != null && cb_DiseaseNumber.SelectedIndex >= 0 && cb_DiseaseNumber.SelectedIndex < selectedSpecies.vDiseases.Count)
                    {
                        Disease selectedDisease = selectedSpecies.vDiseases[cb_DiseaseNumber.SelectedIndex];
                        chckbx_Cured.IsChecked = selectedDisease.bCured != 0;
                        EnableDiseaseUI(selectedDisease.bCured == 1 ? false : true);
                        viewModel.SelectedDisease = selectedDisease;

                        if (selectedDisease.vMedicines.Count > 0)
                        {
                            cb_MedicineNumber.SelectedIndex = 0;
                        }
                        else
                        {
                            viewModel.SelectedMedicine = null;
                        }
                    }
                }
            }
        }

        private void cb_Disease_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                if (cb_Disease.SelectedIndex != -1 && selectedAquarium.vFishSpecies != null && selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].bDiseaseCount > 0)
                {

                    int selected = cb_DiseaseNumber.SelectedIndex;
                    BindingExpression bindingExpression = cb_DiseaseNumber.GetBindingExpression(ComboBox.ItemsSourceProperty);
                    bindingExpression?.UpdateTarget();                   
                    cb_DiseaseNumber.SelectedIndex = selected;
                }
            }
        }

        private void cb_MedicineNumber_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                if (cb_DiseaseSpecies.SelectedItem != null && cb_DiseaseSpecies.SelectedIndex >= 0 && cb_DiseaseSpecies.SelectedIndex < selectedAquarium.vFishSpecies.Count)
                {
                    FishSpecies selectedSpecies = selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex];

                    if (selectedSpecies != null && cb_DiseaseNumber.SelectedIndex >= 0 && cb_DiseaseNumber.SelectedIndex < selectedSpecies.vDiseases.Count)
                    {
                        Disease selectedDisease = selectedSpecies.vDiseases[cb_DiseaseNumber.SelectedIndex];

                        if (selectedDisease != null && cb_MedicineNumber.SelectedIndex >= 0 && cb_MedicineNumber.SelectedIndex < selectedDisease.vMedicines.Count)
                        {
                            Medicine selectedMedicine = selectedDisease.vMedicines[cb_MedicineNumber.SelectedIndex];
                            viewModel.SelectedMedicine = selectedMedicine;
                        }
                    }
                }
            }
        }

        private void cb_Medicine_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                if (selectedAquarium.vFishSpecies != null && cb_DiseaseNumber.SelectedIndex != -1 && selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].vMedicines.Count>0)
                {
                    int selected = cb_MedicineNumber.SelectedIndex;
                    BindingExpression bindingExpression = cb_MedicineNumber.GetBindingExpression(ComboBox.ItemsSourceProperty);
                    bindingExpression?.UpdateTarget();
                    cb_MedicineNumber.SelectedIndex = selected;
                }
            }
        }

        private void chckbx_WaterPresent_Checked(object sender, RoutedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                if (chckbx_WaterPresent.IsChecked == true)
                {
                    selectedAquarium.bWaterPresent = 1;
                }
                else
                {
                    if (selectedAquarium.bSpeciesCount > 0)
                    {
                        MessageBox.Show(TryFindResource("STR_SPECIES_IN_AQUARIUM").ToString());
                        chckbx_WaterPresent.IsChecked = true;
                    }
                    else
                    {
                        selectedAquarium.bWaterPresent = 0;
                    }
                }
            }
        }

        private void chckbx_Cured_Checked(object sender, RoutedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                if (selectedAquarium.vFishSpecies != null && selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases.Count>0)
                {
                    if (chckbx_Cured.IsChecked == true)
                    {
                        selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].bCured = 1;
                        EnableDiseaseUI(false);
                    }
                    else
                    {
                        selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].bCured = 0;
                        EnableDiseaseUI(true);
                    }
                } 
            }
        }

        private void tb_AquariumLastWaterChange_SelectedDateChanged(object sender, SelectionChangedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                selectedAquarium.dNextWaterChange = selectedAquarium.bWaterChangeUnit == 0 ? selectedAquarium.dLastWaterChange.AddDays(selectedAquarium.wWaterChangePeriod) : selectedAquarium.dLastWaterChange.AddHours(selectedAquarium.wWaterChangePeriod);
                tb_AquariumNextWaterChange.Foreground = (selectedAquarium.dNextWaterChange <= DateTime.Today) ? Brushes.Red : Brushes.Black;
            }
        }

        private void tb_AquariumWaterChangePeriod_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                selectedAquarium.dNextWaterChange = selectedAquarium.bWaterChangeUnit == 0 ? selectedAquarium.dLastWaterChange.AddDays(selectedAquarium.wWaterChangePeriod) : selectedAquarium.dLastWaterChange.AddHours(selectedAquarium.wWaterChangePeriod);
                tb_AquariumNextWaterChange.Foreground = (selectedAquarium.dNextWaterChange <= DateTime.Today) ? Brushes.Red : Brushes.Black;
            }
        }

        private void tb_AquariumLastFilterClean_SelectedDateChanged(object sender, SelectionChangedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                selectedAquarium.dNextFilterClean = selectedAquarium.dLastFilterClean.AddDays(selectedAquarium.wFilterCleanPeriod);
                tb_AquariumNextFilterClean.Foreground = (selectedAquarium.dNextFilterClean <= DateTime.Today) ? Brushes.Red : Brushes.Black;
            }
        }

        private void tb_AquariumFilterCleanPeriod_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                selectedAquarium.dNextFilterClean = selectedAquarium.dLastFilterClean.AddDays(selectedAquarium.wFilterCleanPeriod);
                tb_AquariumNextFilterClean.Foreground = (selectedAquarium.dNextFilterClean <= DateTime.Today) ? Brushes.Red : Brushes.Black;
            }
        }

        private void dp_LastDosage_SelectedDateChanged(object sender, SelectionChangedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                if (selectedAquarium.vFishSpecies != null && cb_DiseaseNumber.SelectedIndex != -1 && selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].vMedicines.Count > 0)
                {
                    selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].vMedicines[cb_MedicineNumber.SelectedIndex].dNextDosage = selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].vMedicines[cb_MedicineNumber.SelectedIndex].bDosagePeriodUnit == 0 ?
                    selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].vMedicines[cb_MedicineNumber.SelectedIndex].dLastDosage.AddDays(
                        selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].vMedicines[cb_MedicineNumber.SelectedIndex].wDosagePeriod)
                    : selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].vMedicines[cb_MedicineNumber.SelectedIndex].dLastDosage.AddHours(
                        selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].vMedicines[cb_MedicineNumber.SelectedIndex].wDosagePeriod);
                    tb_NextDosage.Foreground = (selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].vMedicines[cb_MedicineNumber.SelectedIndex].dNextDosage <= DateTime.Today) ? Brushes.Red : Brushes.Black;
                }
            }
        }

        private void tb_DosagePeriod_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {

                if (selectedAquarium.vFishSpecies != null && cb_DiseaseNumber.SelectedIndex != -1 && selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].vMedicines.Count > 0)
                {
                    selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].vMedicines[cb_MedicineNumber.SelectedIndex].dNextDosage = selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].vMedicines[cb_MedicineNumber.SelectedIndex].bDosagePeriodUnit == 0 ?
                    selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].vMedicines[cb_MedicineNumber.SelectedIndex].dLastDosage.AddDays(
                        selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].vMedicines[cb_MedicineNumber.SelectedIndex].wDosagePeriod)
                    : selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].vMedicines[cb_MedicineNumber.SelectedIndex].dLastDosage.AddHours(
                        selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].vMedicines[cb_MedicineNumber.SelectedIndex].wDosagePeriod);
                    tb_NextDosage.Foreground = (selectedAquarium.vFishSpecies[cb_DiseaseSpecies.SelectedIndex].vDiseases[cb_DiseaseNumber.SelectedIndex].vMedicines[cb_MedicineNumber.SelectedIndex].dNextDosage <= DateTime.Today) ? Brushes.Red : Brushes.Black;
                }
            }
        }

        private void tb_NextDosage_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (aquariumListView.SelectedItem != null && aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                CheckAquariumProperty(selectedAquarium);
            }           
        }

        private void tb_Dosage100L_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                if (viewModel.SelectedMedicine != null)
                {
                    CalcDose(selectedAquarium, ushort.Parse(tb_Dosage100L.Text));
                } 
            }
        }

        private void tb_FishMortality_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                if (cb_SpeciesNumber.SelectedIndex >= 0 && cb_SpeciesNumber.SelectedIndex < selectedAquarium.vFishSpecies.Count)
                {
                    FishSpecies selectedSpecies = selectedAquarium.vFishSpecies[cb_SpeciesNumber.SelectedIndex];
                    tb_FishCurrentQuantity.Text = (selectedSpecies.wStockQuantity - selectedSpecies.wMortality).ToString();
                }                   
            }
        }

        #endregion

        #region MenuItems

        private void FISH_SPECIES_MENU_ITEM_Click(object sender, RoutedEventArgs e)
        {
            FishListDialog fishListDialog = new FishListDialog(this);
            fishListDialog.ShowDialog();
        }

        private void DISEASES_MENU_ITEM_Click(object sender, RoutedEventArgs e)
        {
            DiseaseListDialog diseaseListDialog = new DiseaseListDialog(this);
            diseaseListDialog.ShowDialog();
        }

        private void MEDICINES_MENU_ITEM_Click(object sender, RoutedEventArgs e)
        {
            MedicineListDialog medicineListDialog = new MedicineListDialog(this);
            medicineListDialog.ShowDialog();
        }

        private void SUPPLIERS_MENU_ITEM_Click(object sender, RoutedEventArgs e)
        {
            SupplierListDialog supplierListDialog = new SupplierListDialog(this);
            supplierListDialog.ShowDialog();
        }

        private void Daily_report_menu_item_Click(object sender, RoutedEventArgs e)
        {
            DailyReport dailyReport = new DailyReport(this);
            dailyReport.dp_DailyReport.SelectedDate = DateTime.Today.Date;
            dailyReport.ShowDialog();
        }

        private void TEMPERATURE_MENU_ITEM_Click(object sender, RoutedEventArgs e)
        {
            MenuItem menuItem = sender as MenuItem;
            if (menuItem != null && menuItem.IsChecked == false)
            {
                string[] nameParts = menuItem.Name.Split('_');
                string selectedUnit = nameParts[nameParts.Length - 1];

                try
                {
                    WriteRegistryValue("TemperatureUnit", selectedUnit);
                    TemperatureUnit = selectedUnit;
                    lbl_Temperature.Content = (TryFindResource("STR_TEMPERATURE").ToString() + " (" + (selectedUnit == "CELSIUS" ? "°C" : "°F") + "):");
                    SelectMenuItem(MENU_ITEM_TEMPERATURE, TemperatureUnit);
                    foreach (Aquarium a in AquariumList)
                    {
                        a.fTemperature = TemperatureUnit == "CELSIUS" ? FahrenheitToCelsius(a.fTemperature) : CelsiusToFahrenheit(a.fTemperature);
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error: " + ex.Message);
                }
            }
        }
        private void ADD_AQUARIUM_MENU_ITEM_Click(object sender, RoutedEventArgs e)
        {
            AquariumList.Add(new Aquarium());
            LoadListView();
        }

        private void DELETE_AQUARIUM_MENU_ITEM_Click(object sender, RoutedEventArgs e)
        {

            if (aquariumListView.SelectedItem != null)
            {
                Aquarium selectedAquarium = (Aquarium)aquariumListView.SelectedItem;
                AquariumList.Remove(selectedAquarium);
                LoadListView();
            }
        }

        private void Diagnostics_menu_item_Click(object sender, RoutedEventArgs e)
        {
            DiagnosticsWindow diagnosticsWindow = new DiagnosticsWindow();
            string imageFileName = $"{SelectedLanguage}_Diseases.png";
            string imagePath = System.IO.Path.Combine(appDirectory, "Data", "Images", imageFileName);
            diagnosticsWindow.SetDiseaseImage(imagePath);
            diagnosticsWindow.Show();
        }

        #endregion

        #region Searching

        private void tb_Search_GotFocus(object sender, RoutedEventArgs e)
        {
            TextBox textBox = (TextBox)sender;

            if (textBox.Text == TryFindResource("STR_SEARCH_TEXT").ToString())
            {
                textBox.Text = string.Empty;
            }
        }

        private void btn_Search_Click(object sender, RoutedEventArgs e)
        {
            if (tb_Search.Text.Length > 0)
            {
                bool bFound = false;

                for (int i = lastSelectedSearchIndex + 1; i < AquariumList.Count; i++)
                {
                    Aquarium item = AquariumList[i];
                    if (item.strID.Contains((tb_Search.Text).ToLower()) || item.vFishSpecies.Any(species => species.strFishName.ToLower().Contains(tb_Search.Text.ToLower())))
                    {
                        int listViewIndex = i;
                        if (listViewIndex != -1)
                        {
                            aquariumListView.SelectedIndex = listViewIndex;
                            lastSelectedSearchIndex = i;
                            bFound = true;
                        }
                        break;
                    }
                }

                if (!bFound)
                {
                    lastSelectedSearchIndex = -1;
                    MessageBox.Show(TryFindResource("STR_NOT_FOUND_AQUARIUM").ToString());
                }
            }
        }

        #endregion

        #region OtherFunctions
        
        private void ReselectListView()
        {
            int selectedIndex = aquariumListView.SelectedIndex;
            aquariumListView.SelectedIndex = -1;
            aquariumListView.SelectedIndex = selectedIndex;
        }
        

        private float CelsiusToFahrenheit(float temperature)
        {
            return (temperature * 9 / 5) + 32;
        }

        private float FahrenheitToCelsius(float temperature)
        {
            return (temperature - 32) * 5 / 9;
        }

        private void CalcDose(Aquarium selectedAquarium, ushort dosage100)
        {
            tb_DosageCurrent.Text = ((float)(selectedAquarium.fVolume * dosage100) / 100).ToString();
        }

        public string FindUniqueID(List<Aquarium> aquariums)
        {
            int maxID = 0;

            foreach (Aquarium a in aquariums)
            {
                if (int.TryParse(a.strID, out int aquariumID))
                {
                    maxID = Math.Max(maxID, aquariumID);
                }
            }

            return (maxID + 1).ToString();
        }

        private void CheckAquariumProperty(Aquarium a)
        {
            bool HasDisease = false;
            bool HasDosageWarning = false;
            foreach (FishSpecies f in a.vFishSpecies)
            {
                if (f.bDiseaseCount > 0)
                {
                    HasDisease = true;
                }
                foreach (Disease d in f.vDiseases)
                {
                    foreach (Medicine m in d.vMedicines)
                    {
                        if (m.bDosagePeriodUnit == 0)
                        {
                            if (m.dLastDosage.AddDays(m.wDosagePeriod) <= DateTime.Today)
                            {
                                HasDosageWarning = true;
                            }
                        }
                        else
                        {
                            if (m.dLastDosage.AddHours(m.wDosagePeriod) <= DateTime.Today)
                            {
                                HasDosageWarning = true;
                            }
                        }
                    }
                }
            }
            a.bHasDisease = HasDisease;
            a.bHasDosageWarning = HasDosageWarning;
        }

        private void btn_Clear_Aquarium_Click(object sender, RoutedEventArgs e)
        {
            if (aquariumListView.SelectedItem != null && aquariumListView.SelectedItem is Aquarium selectedAquarium)
            {
                selectedAquarium.bSpeciesCount = 0;
                selectedAquarium.vFishSpecies.Clear();
                ReselectListView();
            }
        }

        #endregion

        
    }
}